@@ -1,4 +1,4 @@
-# last modified 21 September 2010 by J. Fox
+# last modified 25 March 2011 by J. Fox
 
 # Data menu dialogs
 
@@ -146,7 +146,7 @@ Recode <- function(){
 				"', as.factor.result=", asFactor, ")", sep="")
 			logger(paste(dataSet,"$",newVar, " <- ", cmd, sep=""))
 			result <- justDoIt(paste(dataSet,"$",newVar, " <- ", cmd, sep=""))
-			if (class(result)[1] !=  "try-error") activeDataSet(dataSet, flushModel=FALSE)
+			if (class(result)[1] !=  "try-error") activeDataSet(dataSet, flushModel=FALSE, flushDialogMemory=FALSE)
 			tkfocus(CommanderWindow())
 		}
 	}
@@ -226,7 +226,7 @@ Compute <- function(){
 			", ", express, ")", sep="")
 		logger(command)
 		result <- justDoIt(command)
-		if (class(result)[1] !=  "try-error") activeDataSet(dataSet, flushModel=FALSE)
+		if (class(result)[1] !=  "try-error") activeDataSet(dataSet, flushModel=FALSE, flushDialogMemory=FALSE)
 		tkfocus(CommanderWindow())
 	}
 	OKCancelHelp(helpSubject="Compute")
@@ -274,7 +274,7 @@ deleteVariable <- function(){
 			eval(parse(text=paste(dataSet, "$", variable, "<- NULL", sep="")), envir=.GlobalEnv)
 			logger(paste(dataSet, "$", variable, " <- NULL", sep=""))
 		}
-		activeDataSet(dataSet, flushModel=FALSE)
+		activeDataSet(dataSet, flushModel=FALSE, flushDialogMemory=FALSE)
 		tkfocus(CommanderWindow())
 	}
 	OKCancelHelp(helpSubject="NULL")
@@ -326,7 +326,7 @@ readDataSet <- function() {
 		location <- tclvalue(locationVariable)
 		file <- if (location == "clipboard") "clipboard" 
 			else if (location == "local") tclvalue(tkgetOpenFile(filetypes=
-							gettextRcmdr('{"Text Files" {".txt" ".TXT" ".dat" ".DAT" ".csv" ".CSV"}} {"All Files" {"*"}}')))
+							gettextRcmdr('{"All Files" {"*"}} {"Text Files" {".txt" ".TXT" ".dat" ".DAT" ".csv" ".CSV"}}')))
 			else {
 				initializeDialog(subdialog, title=gettextRcmdr("Internet URL"))
 				onOKsub <- function(){
@@ -617,7 +617,7 @@ importSPSS <- function() {
 			}
 		}
 		file <- tclvalue(tkgetOpenFile(
-				filetypes=gettextRcmdr('{"SPSS save files" {".sav" ".SAV"}} {"SPSS portable files" {".por" ".POR"}} {"All Files" {"*"}}')))
+				filetypes=gettextRcmdr('{"All Files" {"*"}} {"SPSS portable files" {".por" ".POR"}} {"SPSS save files" {".sav" ".SAV"}}')))
 		if (file == "") {
 			tkfocus(CommanderWindow())
 			return()
@@ -672,7 +672,7 @@ importMinitab <- function() {
 			}
 		}
 		file <- tclvalue(tkgetOpenFile(
-				filetypes=gettextRcmdr('{"Minitab portable files" {".mtp" ".MTP"}} {"All Files" {"*"}}')))
+				filetypes=gettextRcmdr('{"All Files" {"*"}} {"Minitab portable files" {".mtp" ".MTP"}}')))
 		if (file == "") {
 			tkfocus(CommanderWindow())
 			return()
@@ -739,7 +739,7 @@ importSTATA <- function() {
 			}
 		}
 		file <- tclvalue(tkgetOpenFile(
-				filetypes=gettextRcmdr('{"STATA datasets" {".dta" ".DTA"}} {"All Files" {"*"}}')))
+				filetypes=gettextRcmdr('{"All Files" {"*"}} {"STATA datasets" {".dta" ".DTA"}}')))
 		if (file == "") {
 			tkfocus(CommanderWindow())
 			return()
@@ -815,7 +815,7 @@ importRODBCtable <- function(){
 			}
 		}
 		File <- tclvalue(tkgetOpenFile(filetypes = gettextRcmdr(
-					'{"MS Excel file" {*.xls ".XLS"}} {"MS Excel 2007 file" {*.xlsx ".XLSX"}} {"MS Access database" {*.mdb ".MDB"}} {"MS Access 2007 database" {*.accdb ".ACCDB"}} {"dBase-like file" {*.dbf ".DBF"}} {"All Files" {"*"}}'
+					'{"All Files" {"*"}} {"MS Access database" {".mdb" ".MDB"}} {"MS Access 2007 database" {".accdb" ".ACCDB"}} {"dBase-like file" {".dbf" ".DBF"}} {"MS Excel 2007 file" {".xlsx" ".XLSX"}} {"MS Excel file" {".xls" ".XLS"}}'
 				)))
 		if(File == ""){
 			tkfocus(CommanderWindow())
@@ -983,7 +983,7 @@ numericToFactor <- function(){
 				command <- paste("as.factor(", .activeDataSet, "$", name, ")", sep="")
 				result <- justDoIt(paste(.activeDataSet, "$", fname, " <- ", command, sep=""))
 				logger(paste(.activeDataSet, "$", fname," <- ", command, sep=""))
-				if (class(result)[1] !=  "try-error") activeDataSet(.activeDataSet, flushModel=FALSE)
+				if (class(result)[1] !=  "try-error") activeDataSet(.activeDataSet, flushModel=FALSE, flushDialogMemory=FALSE)
 				tkfocus(CommanderWindow())
 			}
 		}
@@ -1088,7 +1088,7 @@ binVariable <- function(){
 			", method=", "'", method, "', labels=", labels, ")", sep="")
 		logger(command)
 		result <- justDoIt(command)
-		if (class(result)[1] !=  "try-error") activeDataSet(.activeDataSet, flushModel=FALSE)
+		if (class(result)[1] !=  "try-error") activeDataSet(.activeDataSet, flushModel=FALSE, flushDialogMemory=FALSE)
 		tkfocus(CommanderWindow())
 	}
 	OKCancelHelp(helpSubject="bin.var")
@@ -1163,7 +1163,7 @@ reorderFactor <- function(){
 				ordered, ")", sep="")
 			result <- justDoIt(paste(.activeDataSet, "$", name, " <- ", command, sep=""))
 			logger(paste(.activeDataSet,"$", name," <- ", command, sep=""))
-			if (class(result)[1] !=  "try-error") activeDataSet(.activeDataSet, flushModel=FALSE)
+			if (class(result)[1] !=  "try-error") activeDataSet(.activeDataSet, flushModel=FALSE, flushDialogMemory=FALSE)
 		}
 		subOKCancelHelp()
 		tkgrid(labelRcmdr(subdialog, text=gettextRcmdr("Old Levels"), fg="blue"),
@@ -1222,7 +1222,7 @@ standardize <- function(X){
 		}
 		remove(.Z, envir=.GlobalEnv)
 		logger("remove(.Z)")
-		if (class(result)[1] !=  "try-error") activeDataSet(.activeDataSet, flushModel=FALSE)
+		if (class(result)[1] !=  "try-error") activeDataSet(.activeDataSet, flushModel=FALSE, flushDialogMemory=FALSE)
 		tkfocus(CommanderWindow())
 	}
 	OKCancelHelp(helpSubject="scale")
@@ -1268,7 +1268,7 @@ exportDataSet <- function() {
 			else if (delim == "spaces") " "
 			else if (delim == "commas") ","
 			else trim.blanks(tclvalue(otherVariable))
-		saveFile <- tclvalue(tkgetSaveFile(filetypes=gettextRcmdr('{"Text Files" {".txt" ".TXT" ".dat" ".DAT" ".csv" ".CSV"}} {"All Files" {"*"}}'),
+		saveFile <- tclvalue(tkgetSaveFile(filetypes=gettextRcmdr('{"All Files" {"*"}} {"Text Files" {".txt" ".TXT" ".dat" ".DAT" ".csv" ".CSV"}}'),
 				defaultextension="txt", initialfile=paste(dsname, ".txt", sep="")))
 		if (saveFile == "") {
 			tkfocus(CommanderWindow())
@@ -1451,7 +1451,7 @@ setCaseNames <- function(){
 		logger(command)
 		eval(parse(text=paste(dataSet, "$", variable, "<- NULL", sep="")), envir=.GlobalEnv)
 		logger(paste(dataSet, "$", variable, " <- NULL", sep=""))
-		if (class(result)[1] !=  "try-error") activeDataSet(dataSet, flushModel=FALSE)
+		if (class(result)[1] !=  "try-error") activeDataSet(dataSet, flushModel=FALSE, flushDialogMemory=FALSE)
 		tkfocus(CommanderWindow())
 	}
 	OKCancelHelp(helpSubject="row.names")
@@ -1505,7 +1505,7 @@ renameVariables <- function(){
 				")] <- c(", paste('"', newnames, '"', collapse=",", sep=""), ")", sep="")
 			result <- justDoIt(command)
 			logger(command)
-			if (class(result)[1] !=  "try-error") activeDataSet(.activeDataSet, flushModel=FALSE)
+			if (class(result)[1] !=  "try-error") activeDataSet(.activeDataSet, flushModel=FALSE, flushDialogMemory=FALSE)
 			tkfocus(CommanderWindow())
 		}
 		subOKCancelHelp()
@@ -1623,7 +1623,7 @@ setContrasts <- function(){
 				logger(command)
 				justDoIt("remove(.Contrasts, envir=.GlobalEnv)")
 				logger("remove(.Contrasts)")
-				if (class(result)[1] !=  "try-error") activeDataSet(ActiveDataSet(), flushModel=FALSE)
+				if (class(result)[1] !=  "try-error") activeDataSet(ActiveDataSet(), flushModel=FALSE, flushDialogMemory=FALSE)
 				tkfocus(CommanderWindow())
 			}
 			subOKCancelHelp(helpSubject="contrasts")
@@ -1654,7 +1654,7 @@ addObsNumbers <- function(){
 	command <- paste(dsname, "$ObsNumber <- 1:", nrows, sep="")
 	logger(command)
 	result <- justDoIt(command)
-	if (class(result)[1] !=  "try-error") activeDataSet(dsname, flushModel=FALSE)
+	if (class(result)[1] !=  "try-error") activeDataSet(dsname, flushModel=FALSE, flushDialogMemory=FALSE)
 }
 
 Stack <- function(){
@@ -1725,7 +1725,7 @@ Stack <- function(){
 
 loadDataSet <- function() {
 	file <- tclvalue(tkgetOpenFile(filetypes=
-				gettextRcmdr('{"R Data Files" {".rda" ".Rda" ".RDA" ".RData"}} {"All Files" {"*"}}')))
+				gettextRcmdr('{"All Files" {"*"}} {"R Data Files" {".RData" ".rda" ".Rda" ".RDA"}}')))
 	if (file == "") return()
 	command <- paste('load("', file,'")', sep="")
 	dsname <- justDoIt(command)
@@ -1736,8 +1736,8 @@ loadDataSet <- function() {
 
 saveDataSet <- function() {
 	file <- tclvalue(tkgetSaveFile(filetypes=
-				gettextRcmdr('{"R Data Files" {".rda" ".Rda" ".RDA" ".RData"}} {"All Files" {"*"}}'),
-			defaultextension="rda", initialfile=paste(activeDataSet(), "rda", sep=".")))
+				gettextRcmdr('{"All Files" {"*"}} {"R Data Files" {".RData" ".rda" ".Rda" ".RDA"}}'),
+			defaultextension=".RData", initialfile=paste(activeDataSet(), ".RData", sep="")))
 	if (file == "") return()
 	command <- paste('save("', activeDataSet(), '", file="', file, '")', sep="")
 	justDoIt(command)
